<?php

namespace Teknisa\PLA\Modules\Parametros\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\Libs\Util\Environment;
use TecFood\Services\User as UserService;
use Zeedhi\Framework\DataSource\FilterCriteria;


class Ger01000UnidadeAdicionalProdutoController  extends \Teknisa\GER\Controllers\TeknisaCrud {
    
    protected $dataSourceName = "ger01000_conversao_unidade";
    protected $connection;

    public function __construct($connection){
        
        $this->connection = $connection;
    }
    
    public function getConversaoUnidade(DTO\Request\Filter $request, DTO\Response $response) {
        $conditions = $request->getFilterCriteria()->getConditions();
        $queryFilter = '';
        $test = true;
        foreach($conditions as $filter){
            if(is_array($filter['value'])){
                $stringToFilter = "'".$filter['value'][0]['_ALL']."'";
                $queryFilter .= ' AND ( (UC.DSUNIDADE LIKE '.strtoupper($stringToFilter).' OR UC.DSUNIDADE LIKE '.strtolower($stringToFilter).') OR (UP.DSUNIDADE LIKE '.strtoupper($stringToFilter).' OR UP.DSUNIDADE LIKE '.strtolower($stringToFilter).'))';
            }else if(isset($filter['value']) && !is_array($filter['value']) && !empty($filter['value'])){
                    $queryFilter .= ' AND CP.'. $filter['columnName'] .' '.$filter['operator'].' '.$filter['value'];
            }
            
        }
		$sql = "SELECT
                    CP.CDPRODUTO,
                    PR.NMPRODUTO,
                    CP.SGUNIDADE,
                    CP.NRORG,
                    UC.DSUNIDADE,
                    ROUND(VRFATOCONVES,5) AS VRFATOCONVES,
                    UP.DSUNIDADE AS DSUNIDPROD
                FROM
                    PRODUTO PR,
                    CONVPRODUTO CP,
                    UNIDMEDIDA UP,
                    UNIDMEDIDA UC
                WHERE
                    ( PR.CDPRODUTO = CP.CDPRODUTO )
                    AND ( PR.SGUNIDADE = UP.SGUNIDADE )
                    AND ( CP.SGUNIDADE = UC.SGUNIDADE )". $queryFilter;
        
		$query = $this->connection->prepare($sql);
        $query->execute();
        $result = $query->fetchAll();
        for($i = 0; $i < sizeof($result); $i++){
            $str = strval($result[$i]['VRFATOCONVES']);
            $str = str_replace(".", ",", $str);
            $result[$i]['VRFATOCONVES'] = $str;
            $result[$i]['NAME'] = ('1 ' . $result[$i]['DSUNIDADE'] . ' equivale a ' . $str . ' ' . $result[$i]['DSUNIDPROD'] . '(s).');
        }
        $response->addDataSet(new DataSet("dataset", $result));
	}
    
    public function saveRegister(DTO\Request $request, DTO\Response $response){
        
        $row = $request->getRow();
        
        if($row["OPCOES_RELACAO"] == 0 ){
        
        $params = array(
            "SGUNIDADE" => $row["SGUNIDADE"],
            "CDPRODUTO" => $row["CDPRODUTO"],
            "VRFATOCONVES" => 1/$row["VRFATOCONVES"]
        );
            
        }
        
        else {
             $params = array(
            "SGUNIDADE" => $row["SGUNIDADE"],
            "CDPRODUTO" => $row["CDPRODUTO"],
            "VRFATOCONVES" => floatval($row["VRFATOCONVES"])
            );
        }
        
        $sqlInsert = "INSERT INTO CONVPRODUTO (SGUNIDADE, CDPRODUTO, VRFATOCONVES) VALUES (:SGUNIDADE, :CDPRODUTO, :VRFATOCONVES)";
        try {
            $std = $this->connection->prepare($sqlInsert);
            $std->execute($params);
        }catch (Exception $e){
            throw $e->getMessage();
        }
    }
    
    public function saveRegisterConversao(DTO\Request $request, DTO\Response $response){
        
        $row = $request->getRow();
        
         if($row["OPCOES_RELACAO"] == 0 ){  
            $params = array(
                "SGUNIDADE" => $row["SGUNIDADE"],
                "CDPRODUTO" => $row["CDPRODUTO"],
                "VRFATOCONVES" => 1/floatval($row["VRFATOCONVES"])
            );
        }
        else {
             $params = array(
            "SGUNIDADE" => $row["SGUNIDADE"],
            "CDPRODUTO" => $row["CDPRODUTO"],
            "VRFATOCONVES" => floatval($row["VRFATOCONVES"])
            );
        }
        
        $sqlInsert = "INSERT INTO CONVPRODUTO (SGUNIDADE, CDPRODUTO, VRFATOCONVES) VALUES (:SGUNIDADE, :CDPRODUTO, :VRFATOCONVES)";
        try {
            $std = $this->connection->prepare($sqlInsert);
            $std->execute($params);
        } catch ( \Exception $e){
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
    }
    
    public function updateConversao(DTO\Request $request, DTO\Response $response){
        
        $row = $request->getRow();
        
        if($row["OPCOES_RELACAO"] == 0 ){
        
        $params = array(
            "SGUNIDADE" => $row["SGUNIDADE"],
            "CDPRODUTO" => $row["CDPRODUTO"],
            "VRFATOCONVES" => 1/floatval($row["VRFATOCONVES"])
        );
            
        }
        
        else {
             $params = array(
            "SGUNIDADE" => $row["SGUNIDADE"],
            "CDPRODUTO" => $row["CDPRODUTO"],
            "VRFATOCONVES" => floatval($row["VRFATOCONVES"])
            );
        }

        $sqlUpdate = "UPDATE CONVPRODUTO SET VRFATOCONVES =:VRFATOCONVES WHERE SGUNIDADE =:SGUNIDADE AND CDPRODUTO =:CDPRODUTO";
        
        try {
            $std = $this->connection->prepare($sqlUpdate);
            $std->execute($params);
        } catch (Exception $e){
            throw $e;
        }
    }
    
    
    

    public function editConversao(DTO\Request $request, DTO\Response $response){
        $row = $request->getRow();
        if ($row["__is_new"]){
            $this->saveRegisterConversao($request, $response);
        } else {
            
            $this->updateConversao($request, $response);
            
        }
    }
    
    public function verificaProduto($row){
    
        $params = array(
            "P_CDPRODUTO" => $row["CDPRODUTO"]    
        );
        
        $sqlDelete = "SELECT R.CDPRATO, 'P' AS TIPO, '' AS CDFILIAL, '' AS CDSERVFILI FROM RECEPADR R WHERE CDPRODUTO = :P_CDPRODUTO AND EXISTS(SELECT SGUNIDADE FROM CONVPRODUTO C WHERE C.CDPRODUTO = R.CDPRODUTO AND C.SGUNIDADE = R.SGUNIDADE) UNION ALL SELECT R.CDPRATO, 'F' AS TIPO, R.CDFILIAL, '' AS CDSERVFILI FROM RECEFILI R WHERE CDPRODUTO = :P_CDPRODUTO AND EXISTS(SELECT SGUNIDADE FROM CONVPRODUTO C WHERE C.CDPRODUTO = R.CDPRODUTO AND C.SGUNIDADE = R.SGUNIDADE) UNION ALL SELECT R.CDPRATO, 'R' AS TIPO, R.CDFILIAL, R.CDSERVFILI FROM RECESERV R WHERE CDPRODUTO = :P_CDPRODUTO AND EXISTS(SELECT SGUNIDADE FROM CONVPRODUTO C WHERE C.CDPRODUTO = R.CDPRODUTO AND C.SGUNIDADE = R.SGUNIDADE) ORDER BY 1";
       
        try {
            $std = $this->connection->prepare($sqlDelete);
            $std->execute($params);
            $result = $std->fetchAll();
            return $result;
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function deleteProduto(DTO\Request $request, DTO\Response $response){
        
        $row = $request->getRow();
        $params = array(
            "P_CDPRODUTO" => $row["CDPRODUTO"]    
        );
        
        $sqlDelete = "DELETE FROM CONVPRODUTO WHERE (CDPRODUTO = :P_CDPRODUTO)";
       
        $produto = $this->verificaProduto($row);
       
        if(!$produto){  
          $std = $this->connection->prepare($sqlDelete);
          $std->execute($params);
        } else {
            $message = "produtoUtilizadoPratos";
            $pratos = '';
            foreach($produto as $key => $value){
                $pratos .= $value['CDPRATO'];
                $pratos .= $key+1==sizeof($produto) ? '.' : ', ';
            }
            $response->addDataSet(new DataSet("errorInfo", array("PRATOS" => $pratos)));
            $response->setError(new Error($message, 500));
        }
    }
    
    public function verificaProdutoConversao($row){
    
        $params = array(
            "P_CDPRODUTO" => $row["CDPRODUTO"],   
            "P_SGUNIDADE" => $row["SGUNIDADE"]
        );
        
        $sqlDelete = "SELECT R.CDPRATO, 'P' AS TIPO, '' AS CDFILIAL, '' AS CDSERVFILI FROM RECEPADR R WHERE CDPRODUTO = :P_CDPRODUTO AND SGUNIDADE = :P_SGUNIDADE AND EXISTS(SELECT SGUNIDADE FROM CONVPRODUTO C WHERE C.CDPRODUTO = R.CDPRODUTO AND C.SGUNIDADE = R.SGUNIDADE) UNION ALL SELECT R.CDPRATO, 'F' AS TIPO, R.CDFILIAL, '' AS CDSERVFILI FROM RECEFILI R WHERE CDPRODUTO = :P_CDPRODUTO AND SGUNIDADE = :P_SGUNIDADE AND EXISTS(SELECT SGUNIDADE FROM CONVPRODUTO C WHERE C.CDPRODUTO = R.CDPRODUTO AND C.SGUNIDADE = R.SGUNIDADE) UNION ALL SELECT R.CDPRATO, 'R' AS TIPO, R.CDFILIAL, R.CDSERVFILI FROM RECESERV R WHERE CDPRODUTO = :P_CDPRODUTO AND SGUNIDADE = :P_SGUNIDADE AND EXISTS(SELECT SGUNIDADE FROM CONVPRODUTO C WHERE C.CDPRODUTO = R.CDPRODUTO AND C.SGUNIDADE = R.SGUNIDADE) ORDER BY 1";
       
        try {
            $std = $this->connection->prepare($sqlDelete);
            $std->execute($params);
            $result = $std->fetchAll();
            return $result;
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function deleteProdutoConversao(DTO\Request $request, DTO\Response $response){
        
        $row = $request->getRow();
        $params = array(
            "OLD_CDPRODUTO" => $row["CDPRODUTO"],   
            "OLD_SGUNIDADE" => $row["SGUNIDADE"]  
        );
        
        $sqlDelete = "DELETE FROM CONVPRODUTO WHERE SGUNIDADE = :OLD_SGUNIDADE AND CDPRODUTO = :OLD_CDPRODUTO";
       
        $produto = $this->verificaProdutoConversao($row);
       
        if(!$produto){  
          $std = $this->connection->prepare($sqlDelete);
          $std->execute($params);
        } else {
            $message = "produtoUtilizadoPratos";
            $pratos = "";
            foreach($produto as $key => $value){
                $pratos .= $value['CDPRATO'];
                $pratos .= $key+1==sizeof($produto) ? '.' : ', ';
            }
            $response->addDataSet(new DataSet("errorInfo", array("PRATOS" => $pratos)));
            $response->setError(new Error($message, 500));
        }
    }
}